@@ -1,24 +1,29 @@
 #ifndef R2_LIB_H
 #define R2_LIB_H
 
-// TODO: rename type from int to 4 byte string
-// TODO: use 4 chars to idnetify plugin type
-
 #include "r_types.h"
 #include "r_list.h"
 
+#if __UNIX__
+#include <dlfcn.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 R_LIB_VERSION_HEADER (r_lib);
+
 // rename to '.' ??
 #define R_LIB_SEPARATOR "."
+#define R_LIB_SYMNAME "radare_plugin"
+#define R_LIB_SYMFUNC "radare_plugin_function"
 
-#define R_LIB_ENV "LIBR_PLUGINS"
+#define R_LIB_ENV "R2_LIBR_PLUGINS"
 
-/* XXX : This must depend on HOST_OS */
+/* TODO: This must depend on HOST_OS, and maybe move into r_types */
 #if __WINDOWS__
+#include <windows.h>
 #define R_LIB_EXT "dll"
 #elif __APPLE__
 #define R_LIB_EXT "dylib"
@@ -35,12 +40,13 @@ typedef struct r_lib_plugin_t {
 	void *dl_handler; // DL HANDLER
 	char *author;
 	char *version;
+	void (*free)(void *data);
 } RLibPlugin;
 
 /* store list of initialized plugin handlers */
 typedef struct r_lib_handler_t {
 	int type;
-	char desc[128];
+	char desc[128]; // TODO: use char *
 	void *user; /* user pointer */
 	int (*constructor)(RLibPlugin *, void *user, void *data);
 	int (*destructor)(RLibPlugin *, void *user, void *data);
@@ -52,8 +58,12 @@ typedef struct r_lib_struct_t {
 	int type;
 	void *data; /* pointer to data handled by plugin handler */
 	const char *version; /* r2 version */
+	void (*free)(void *data);
+	const char *pkgname; /* pkgname associated to this plugin */
 } RLibStruct;
 
+typedef RLibStruct* (*RLibStructFunc) (void);
+
 // order matters because of libr/util/lib.c
 enum {
 	R_LIB_TYPE_IO,      /* io layer */
@@ -62,8 +72,9 @@ enum {
 	R_LIB_TYPE_ASM,     /* assembler */
 	R_LIB_TYPE_ANAL,    /* analysis */
 	R_LIB_TYPE_PARSE,   /* parsers */
-	R_LIB_TYPE_BIN,     /* bins */
+	R_LIB_TYPE_BIN,     /* bin headers */
 	R_LIB_TYPE_BIN_XTR, /* bin extractors */
+	R_LIB_TYPE_BIN_LDR, /* bin loaders */
 	R_LIB_TYPE_BP,      /* breakpoint */
 	R_LIB_TYPE_SYSCALL, /* syscall */
 	R_LIB_TYPE_FASTCALL,/* fastcall */
@@ -71,6 +82,7 @@ enum {
 	R_LIB_TYPE_CORE,    /* RCore commands */
 	R_LIB_TYPE_EGG,     /* r_egg plugin */
 	R_LIB_TYPE_FS,      /* r_fs plugin */
+	R_LIB_TYPE_ESIL,    /* r_anal.esil plugin */
 	R_LIB_TYPE_LAST
 };
 
@@ -78,33 +90,33 @@ typedef struct r_lib_t {
 	/* linked list with all the plugin handler */
 	/* only one handler per handler-id allowed */
 	/* this is checked in add_handler function */
-	char symname[32];
+	char *symname;
+	char *symnamefunc;
 	RList /*RLibPlugin*/ *plugins;
 	RList /*RLibHandler*/ *handlers;
+	bool ignore_version;
 } RLib;
 
 #ifdef R_API
 /* low level api */
 R_API void *r_lib_dl_open(const char *libname);
+
 R_API void *r_lib_dl_sym(void *handler, const char *name);
 R_API int r_lib_dl_close(void *handler);
-R_API int r_lib_dl_check_filename(const char *file);
 
 /* high level api */
-R_API RLib *r_lib_new(const char *symname);
-R_API RLib *r_lib_free(RLib *lib);
+typedef int (*RLibCallback)(RLibPlugin *, void *, void *);
+R_API RLib *r_lib_new(const char *symname, const char *symnamefunc);
+R_API void r_lib_free(RLib *lib);
 R_API int r_lib_run_handler(RLib *lib, RLibPlugin *plugin, RLibStruct *symbol);
 R_API RLibHandler *r_lib_get_handler(RLib *lib, int type);
 R_API int r_lib_open(RLib *lib, const char *file);
-R_API int r_lib_opendir(RLib *lib, const char *path);
+R_API bool r_lib_opendir(RLib *lib, const char *path);
 R_API int r_lib_open_ptr (RLib *lib, const char *file, void *handler, RLibStruct *stru);
 R_API char *r_lib_path(const char *libname);
 R_API void r_lib_list(RLib *lib);
-R_API int r_lib_add_handler(RLib *lib, int type, const char *desc,
-	int (*cb)(RLibPlugin *, void *, void *),
-	int (*dt)(RLibPlugin *, void *, void *),
-	void *user );
-R_API int r_lib_del_handler(RLib *lib, int type);
+R_API bool r_lib_add_handler(RLib *lib, int type, const char *desc, RLibCallback ct, RLibCallback dt, void *user);
+R_API bool r_lib_del_handler(RLib *lib, int type);
 R_API int r_lib_close(RLib *lib, const char *file);
 
 R_API const char *r_lib_types_get(int idx);
