@@ -3,30 +3,36 @@
 
 #include "r_version.h"
 
+#define R_CHECKS_LEVEL 2
 #define DEBUGGER 1
 #define HAVE_DECL_ADDR_NO_RANDOMIZE 1
 #define HAVE_ARC4RANDOM_UNIFORM 0
 #define HAVE_EXPLICIT_BZERO 1
+#define HAVE_EXPLICIT_MEMSET 0
+#define HAVE_CLOCK_NANOSLEEP 1
+#define HAVE_SIGACTION 1
 
 #ifdef R_MESON_VERSION
 #define R2_PREFIX "/usr"
+#define R2_ETCDIR "/usr/etc"
 #define R2_LIBDIR "/usr/lib"
-#define R2_INCDIR "/usr/include"
+#define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "@WWWROOT@"
-#define R2_PLUGINS "@PLUGINS@"
-#define R2_EXTRAS "@EXTRAS@"
-#define R2_BINDINGS "@BINDINGS@"
-#define R2_DATDIR_R2 "@DATADIR_R2@"
-#define R2_SDB "@SDB@"
-#define R2_ZIGNS "@ZIGNS@"
-#define R2_THEMES "@THEMES@"
-#define R2_FORTUNES "@FORTUNES@"
-#define R2_FLAGS "@FLAGS@"
-#define R2_HUD "@HUD@"
+#define R2_WWWROOT "/usr/share/radare2/5.2.1/www"
+#define R2_PLUGINS "lib/radare2/5.2.1"
+#define R2_EXTRAS "lib/radare2-extras/5.2.1"
+#define R2_BINDINGS "lib/radare2-bindings/5.2.1"
+#define R2_DATDIR_R2 "share/radare2"
+#define R2_SDB "share/radare2/5.2.1"
+#define R2_ZIGNS "share/radare2/5.2.1/zigns"
+#define R2_THEMES "share/radare2/5.2.1/cons"
+#define R2_FORTUNES "share/doc/radare2"
+#define R2_FLAGS "share/radare2/5.2.1/flag"
+#define R2_HUD "share/radare2/5.2.1/hud"
 #else
-#if __WINDOWS__ || __CYGWIN__ || MINGW32 || _MSC_VER || CUTTER
+#if __WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
+#define R2_ETCDIR "etc"
 #define R2_LIBDIR "lib"
 #define R2_INCDIR "include\\libr"
 #define R2_DATDIR "share"
@@ -36,8 +42,9 @@
 #define R2_BINDINGS "lib\\bindings"
 #else
 #define R2_PREFIX "/usr"
+#define R2_ETCDIR "/usr/etc"
 #define R2_LIBDIR "/usr/lib"
-#define R2_INCDIR "/usr/include/libr"
+#define R2_INCDIR "/usr/include/libr/libr"
 #define R2_DATDIR "/usr/share"
 #define R2_WWWROOT R2_DATDIR "/radare2/" R2_VERSION "/www"
 #define R2_PLUGINS "lib/radare2/" R2_VERSION
@@ -68,6 +75,7 @@
 #define R2_HOME_PDB         R_JOIN_2_PATHS (R2_HOME_DATADIR, "pdb")
 #define R2_HOME_PROJECTS    R_JOIN_2_PATHS (R2_HOME_DATADIR, "projects")
 #define R2_HOME_WWWROOT     R_JOIN_2_PATHS (R2_HOME_DATADIR, "www")
+#define R2_HOME_SDB_FCNSIGN R_JOIN_2_PATHS (R2_HOME_DATADIR, "fcnsign")
 #define R2_HOME_SDB_FORMAT  R_JOIN_2_PATHS (R2_HOME_DATADIR, "format")
 #define R2_HOME_HUD         R_JOIN_2_PATHS (R2_HOME_DATADIR, "hud")
 #define R2_HOME_BINRC       R_JOIN_2_PATHS (R2_HOME_DATADIR, "rc.d")
@@ -80,15 +88,20 @@
 #define R2_GLOBAL_RC        R_JOIN_2_PATHS (R2_DATDIR_R2, "radare2rc")
 
 #define HAVE_LIB_MAGIC 1
-#define USE_LIB_MAGIC 0
-#define HAVE_LIB_XXHASH 0
-#define USE_LIB_XXHASH 0
+#define USE_LIB_MAGIC 1
+#define HAVE_LIB_XXHASH 1
+#define USE_LIB_XXHASH 1
+
 #ifndef HAVE_LIB_SSL
-#define HAVE_LIB_SSL 1
+#define HAVE_LIB_SSL 0
 #endif
 
-#define HAVE_FORK 1
+#define HAVE_LIBUV 1
 
+#define HAVE_PTRACE 1
+#define USE_PTRACE_WRAP 1
+#define HAVE_FORK 1
+#define WANT_DYLINK 1
 #define WITH_GPL 1
 
 #if __APPLE__ && __POWERPC__
